import { Entity, Plugin, Viewer } from "ct-g-xeokit-viewer/types";
import { AngleMeasurement } from "./AngleMeasurement";
import { AngleMeasurementsControl } from "./AngleMeasurementsControl";

export declare type AngleMeasurementsPluginConfiguration = {
  /** Optional ID for this plugin, so that we can find it within {@link Viewer.plugins}. */
  id?: string;
  /** Container DOM element for markers and labels. Defaults to ````document.body````. */
  container?: HTMLElement;
  /** The default color of the dots, wire and label. */
  defaultColor?: string;
  /** If set, the wires, dots and labels will have this zIndex (+1 for dots and +2 for labels). */
  zIndex?: number;
};

/**
 * {@link Viewer} plugin for measuring angles.
 */
export declare class AngleMeasurementsPlugin extends Plugin {
  /**
   * @constructor
   * @param {Viewer} viewer The Viewer.
   * @param {AngleMeasurementsPluginConfiguration} [cfg]  Plugin configuration.
  */
  constructor(viewer: Viewer, cfg?: AngleMeasurementsPluginConfiguration);

  /**
   * Gets the {@link AngleMeasurementsControl}, which creates {@link AngleMeasurement}s from user input.
   *
   * @type {AngleMeasurementsControl}
   */
  get control(): AngleMeasurementsControl;

  /**
   * Gets the existing {@link AngleMeasurement}s, each mapped to its {@link AngleMeasurement.id}.
   *
   * @type {{String:AngleMeasurement}}
   */
  get measurements(): { [key:string]: AngleMeasurement};

  /**
   * Creates an {@link AngleMeasurement}.
   *
   * The AngleMeasurement is then registered by {@link AngleMeasurement.id} in {@link AngleMeasurementsPlugin.measurements}.
   *
   * @param {Object} params {@link AngleMeasurement} configuration.
   * @param {String} params.id Unique ID to assign to {@link AngleMeasurement.id}. The AngleMeasurement will be registered by this in {@link AngleMeasurementsPlugin.measurements} and {@link Scene.components}. Must be unique among all components in the {@link Viewer}.
   * @param {Number[]} params.origin.worldPos Origin World-space 3D position.
   * @param {Entity} params.origin.entity Origin Entity.
   * @param {Number[]} params.corner.worldPos Corner World-space 3D position.
   * @param {Entity} params.corner.entity Corner Entity.
   * @param {Number[]} params.target.worldPos Target World-space 3D position.
   * @param {Entity} params.target.entity Target Entity.
   * @param {Boolean} [params.visible=true] Whether to initially show the {@link AngleMeasurement}.
   * @returns {AngleMeasurement} The new {@link AngleMeasurement}.
   */
  createMeasurement(params?: {
      id: string;
      origin: {
        entity: Entity;
        worldPos: number[];
      },
      corner: {
        entity: Entity;
        worldPos: number[];
      },
      target: {
        entity: Entity;
        worldPos: number[];
      },
      visible?: boolean;
  }): AngleMeasurement;

  /**
   * Destroys a {@link AngleMeasurement}.
   *
   * @param {String} id ID of AngleMeasurement to destroy.
   */
  destroyMeasurement(id: string): void;
  
  /**
   * Destroys all {@link AngleMeasurement}s.
   */
  clear(): void;

  /**
   * Fires when a measurement is created.
   * @param {String} event The event
   * @param {Function} callback Callback fired on the event
   * @return {String} Handle to the subscription, which may be used to unsubscribe with {@link #off}.
   */
  on(event: "measurementCreated" | "measurementDestroyed", callback: (measurement: AngleMeasurement)=> void): string;

  /**
   * Fires when a mouse event is happened on measurement.
   * @param {String} event The event
   * @param {Function} callback Callback fired on the event
   * @return {String} Handle to the subscription, which may be used to unsubscribe with {@link #off}.
   */
  on(event: "mouseOver" | "mouseLeave" | "contextMenu", callback: ({ event, measurement }: { event: MouseEvent, measurement: AngleMeasurement })=> void): string;

  /**
   * Cancels an event subscription that was previously made with {@link #on}.
   * @param {String} subId Subscription ID
   */
  off(subId: string): void;
}